/* -*- Mode: C++; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright 2022-Present Couchbase, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 * ANY KIND, either express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

#pragma once

#include "utils/movable_function.hxx"

#include "resource_units.hxx"

#include <chrono>
#include <optional>
#include <string>
#include <system_error>
#include <vector>

namespace couchbase
{
class retry_strategy;
namespace tracing
{
class request_span;
} // namespace tracing
} // namespace couchbase

namespace couchbase::core
{
class get_collection_manifest_options
{
public:
  std::shared_ptr<couchbase::retry_strategy> retry_strategy{};
  std::chrono::milliseconds timeout{};
  std::shared_ptr<couchbase::tracing::request_span> parent_span{};

  struct {
    std::string user{};
  } internal{};
};

class get_collection_manifest_result
{
public:
  std::vector<std::byte> manifest{};

  struct {
    std::optional<resource_unit_result> resource_units{};
  } internal{};
};

using get_collection_manifest_callback =
  utils::movable_function<void(get_collection_manifest_result result, std::error_code ec)>;

class get_collection_id_options
{
public:
  std::shared_ptr<couchbase::retry_strategy> retry_strategy{};
  std::chrono::milliseconds timeout{};
  std::shared_ptr<couchbase::tracing::request_span> parent_span{};

  struct {
    std::string user{};
  } internal{};
};

class get_collection_id_result
{
public:
  std::uint64_t manifest_id{};
  std::uint32_t collection_id{};

  struct {
    std::optional<resource_unit_result> resource_units{};
  } internal{};
};

using get_collection_id_callback =
  utils::movable_function<void(get_collection_id_result result, std::error_code ec)>;
} // namespace couchbase::core
